const copyURL_urlDetectionShortFormat = /^((?:http:|https:|ftp:)\/\/\S+)$/i;
const copyURL_urlDetectionLongFormat = /((?:http:|https:|ftp:)\/\/\S+[\w\/\\\-])/i;
const copyURL_imgURLDetectionFormat = /\.(gif|jpg|jpeg|jpe|png)$/i;

var CopyURL = {
    stringBundleName : "copyurl-strings",

    LinkFormat : null,
    ImageFormat : null,
    SelectedFormat : null,
    
    $ : function(id)
    {
        return CopyURLLib.$(id, null);
    },

    $ : function(id, doc)
    {
        return CopyURLLib.$(id, doc);
    },

    $$ : function(selector)
    {
        return CopyURLLib.getElementsBySelector(content.document, selector);
    },

    onLoad : function() {
        this.LinkFormat = this.GetPreference("linkFormat");
        this.ImageFormat = this.GetPreference("imageFormat");
        this.SelectedFormat = this.GetPreference("selectedFormat");

        this.BuildToolbarFormattingMenu();
        this.UpdateFormatting();
        
        this.$("contentAreaContextMenu").addEventListener("popupshowing", CopyURL.UpdateContextMenu, true);
    },

    // Log() - Log debug messages to the Javascript console. NOTE: javascript.options.showInConsole must be enabled.
    Log : function (message)
    {
        CopyURLUtilities.Log(String.prototype.format.apply(message, Array.slice(arguments, 1)));
    },

    PreferenceWatcher : {
        prefs : null,
        
        Startup : function () {
            this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefService)
                .getBranch(CopyURLPrefs.preferencePrefix);
            this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
            this.prefs.addObserver("", this, false);
        },
        
        // Change what's displayed if a preference changes, and we don't want them to have to reload.
        observe : function(subject, topic, data) {
            if(topic != "nsPref:changed") {
                return;
            }
            
            switch(data) {
                case "formatCount" :
                case "selectedFormat" : {
                    CopyURL.SelectedFormat = CopyURL.GetPreference("selectedFormat");
                    CopyURL.UpdateFormatting();
                    break;
                }
                case "showContextMenuItem" : {
                    CopyURL.UpdateContextMenu();
                    break;
                }
            }
        },
        
        Shutdown : function() {
            this.prefs.removeObserver("", this);
        },
    },
    
    UpdateContextMenu : function() {
        var showContextMenuItem = CopyURL.GetPreference("showContextMenuItem");
        CopyURL.SetVisibility(showContextMenuItem, "miCopyURL");
    },
    
    SetEnabled : function(enabled) {
        var elementIDs = Array.slice(arguments, 1);
        var elements = CopyURL.GetElementsByIDs(elementIDs);

        CopyURLUtilities.each(function(element) {
            element.setAttribute("disabled", !enabled);
        }, elements);
    },

    SetLabel : function(elementID, label) {
        var labelText = String.prototype.format.apply(label, Array.slice(arguments, 2))

        CopyURL.$(elementID).setAttribute("label", labelText);
    },

    SetCheckBoxes : function(checked) {
        var elementIDs = Array.slice(arguments, 1);
        var elements = CopyURL.GetElementsByIDs(elementIDs);

        CopyURLUtilities.each(function(element) {
            element.setAttribute("checked", checked);
        }, elements);
    },

    SetVisibility : function(visible) {
        var elementIDs = Array.slice(arguments, 1);
        var elements = CopyURL.GetElementsByIDs(elementIDs);

        CopyURLUtilities.each(function(element) { 
            element.hidden = !visible;
        }, elements);
    },
    
    GetElementsByIDs : function(elementIDs, doc) {
        return CopyURLUtilities.map(function(elementID) {
            return CopyURL.$(elementID, doc);
        }, elementIDs);
    },
    
    // UTILITY FUNCTIONS
    // -----------------
    // SetPreference(preferenceName, value) - Magically set a preference
    SetPreference : function(preferenceName, value) {
        try {
            CopyURLPrefs.SetPreference(preferenceName, value);
        }
        catch (e) {
            this.Log("Unable to set preference ({0}) to value ({1})! Exception: {2}", preferenceName, value, e);
        }
    },

    // GetPreference(preferenceName) - Magically get a preference
    GetPreference : function(preferenceName) {
        try {
            return CopyURLPrefs.GetPreference(preferenceName);
        }
        catch (e) {
            this.Log("Unable to get preference ({0})! Exception: {1}", preferenceName, e);
        }
    },

    // GetString() - Loads a string resource from the string bundle.
    GetString : function(stringResource) {
        if(null == this.strings)
        {
            this.strings = this.$(this.stringBundleName);
        }

        if(null == this.strings)
        {
            this.Log("Unable to load string bundle! (Tried: {0})", this.stringBundleName);
        }

        if(null == stringResource) return null;

        return this.strings.getString(stringResource);
    },
    
    // GetCurrentURL() - Gets the URL of the current page and makes sure it's valid
    GetCurrentURL : function() {
        var currentURL = "{0}".format(content.document.location.href);

        // Did we get anything?
        if (currentURL.is_blank()) {
            this.Log(this.GetString("errorNoURL"));
            return null;
        }

        // Check if text is completely an URL
        var matches = copyURL_urlDetectionShortFormat.exec(currentURL);

        if (!matches) {
            // Look for an URL embedded in the text.
            matches = copyURL_urlDetectionLongFormat.exec(currentURL);
            
            if (!matches) {
                this.Log(this.GetString("errorNoURL"));
                return null;
            }
        }

        return matches[1];
    },
    
    PromptForInput : function(question, title, defaultValue) {
        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        var input = {value: defaultValue};
        var check = {value: false};

        var result = prompts.prompt(window, title, question, input, null, check);

        if(true == result) return input.value;
        
        return "";
    },

    ToggleBooleanPreference : function(preferenceName) {
        var prefValue = this.GetPreference(preferenceName);
        
        this.SetPreference(preferenceName, !prefValue);
        
        return prefValue;
    },

    // ACTUAL FUNCTIONALITY
    // --------------------
    BuildToolbarFormattingMenu : function(menu) {
        if(null == menu) return;
        
        var formatCount = CopyURLPrefs.GetPreference("formatCount");
        var selectedFormat = CopyURLPrefs.GetPreference("selectedFormat");
        
        this.SetVisibility(formatCount > 0, "copyURL-formatSep");
        
        this.RemoveGeneratedMenuItems(menu);

        var menuButton = this.$("copyURL-button");
        var separator = this.$("copyURL-formatSep");

        // Loop through the defined formats
        for(var i = 1; i <= formatCount; i++) {
            var name = CopyURLPrefs.GetPreference("formats.{0}.name".format(i));
            var accesskey = CopyURLPrefs.GetPreference("formats.{0}.accesskey".format(i));
            var linkformat = CopyURLPrefs.GetPreference("formats.{0}.linkformat".format(i));
            var imageformat = CopyURLPrefs.GetPreference("formats.{0}.imageformat".format(i));
            
            var menuItem = document.createElement("menuitem");
            
            menuItem.setAttribute("label", name);
            menuItem.setAttribute("class", "copyURL-generatedMenuItem");
            menuItem.setAttribute("accesskey", accesskey);
            menuItem.setAttribute("tooltiptext", this.GetString("switchFormattingFormat").format(name));
            menuItem.setAttribute("oncommand", "CopyURL.SwitchFormatting({0}); event.stopPropagation();".format(i));
            menuItem.setAttribute("type", "checkbox");

            if(i == selectedFormat) {
                menuButton.setAttribute("tooltiptext", this.GetString("copyURLTooltipFormat").format(name));
                menuItem.setAttribute("checked", true);
            }

            menu.insertBefore(menuItem, separator);
        }
    },
    
    // Removes all the generated menu items from the menu
    RemoveGeneratedMenuItems : function(menu) {
        var generatedMenuItems = new Array();
        var menuItem = null;
        var menuItems = menu.childNodes;
        var menuItemsLength = menuItems.length;
        
        // Loop through the menu items
        for(var i = 0; i < menuItemsLength; i++) {
            menuItem = menuItems.item(i);

            // If this is a generated menu item
            if(menuItem && menuItem.hasAttribute("class") && menuItem.getAttribute("class") == "copyURL-generatedMenuItem") {
                generatedMenuItems.push(menuItem);
            }
        }

        menuItemsLength = generatedMenuItems.length;

        // Loop through the generated menu items
        for(i = 0; i < menuItemsLength; i++) {
            menu.removeChild(generatedMenuItems[i]);
        }
    },

    SwitchFormatting : function(formatID) {
        var formatCount = this.GetPreference("formatCount");
        
        if (formatID < 1 || formatCount < formatID)
        {
            return;
        }

        this.SetPreference("selectedFormat", formatID);
    },
    
    UpdateFormatting : function() {
        var formatID = this.SelectedFormat;
        
        var name = CopyURLPrefs.GetPreference("formats.{0}.name".format(formatID));
        
        this.LinkFormat = CopyURLPrefs.GetPreference("formats.{0}.linkformat".format(formatID));
        this.ImageFormat = CopyURLPrefs.GetPreference("formats.{0}.imageformat".format(formatID));
        
        var menuButton = this.$("copyURL-button");        
        var contextMenuButton = this.$("miCopyURL");
        
        var copyURLFormatLabel = this.GetString("copyURLTooltipFormat").format(name);
        
        menuButton.setAttribute("tooltiptext", copyURLFormatLabel);
        contextMenuButton.setAttribute("label", copyURLFormatLabel);
    },
    
    EditFormats : function() {
        window.openDialog("chrome://copyurl/content/CopyURLOptions.xul", "copyurl_options", "centerscreen,chrome,modal", null);
    },
    
    CopyFormattedURLToClipboard : function() {
        var url = this.GetCurrentURL();
        
        if(null == url || url.is_blank()) {
            alert(this.GetString("errorNoURL"));
            return;
        }
        
        function concatenateTitles(x, y){
            return x + " " + y.textContent;
        };
        
        var title = CopyURLUtilities.reduce(concatenateTitles, this.$$("title"), "").trim();

        var isImage = copyURL_imgURLDetectionFormat.test(url);
        var format = isImage ? this.ImageFormat : this.LinkFormat;
        
        if(!isImage && title.is_blank()) {
            title = this.PromptForInput(this.GetString("linkTextPrompt"), this.GetString("linkTextTitle"), "");
        }
        
        var clipboardContent = format.replace(/#{URL}/gi, url).replace(/#{TITLE}/gi, title);
        
        CopyURLClipboard.SetClipboardContents(clipboardContent);
    },
};

window.addEventListener("load", function(e) { CopyURL.onLoad(e); }, false);
window.addEventListener("load", function(e) { CopyURL.PreferenceWatcher.Startup(); }, false);
window.addEventListener("unload", function(e) { CopyURL.PreferenceWatcher.Shutdown(); }, false);