/* See firebug_license.txt for terms of usage */

var CopyURLLib = {
    // ************************************************************************************************
    // CSS classes
    test : "TEST",

    hasClass : function(node, name)
    {
        if (!node || node.nodeType != 1)
            return false;
        else
        {
            var re = new RegExp("(^|\\s)"+name+"($|\\s)");
            return re.exec(node.getAttribute("class")) != null;
        }
    },

    setClass : function(node, name)
    {
        if (node && !this.hasClass(node, name))
            node.className += " " + name;
    },

    removeClass : function(node, name)
    {
        if (node && node.className)
        {
            var index = node.className.indexOf(name);
            if (index >= 0)
            {
                var size = name.length;
                node.className = node.className.substr(0,index-1) + node.className.substr(index+size);
            }
        }
    },

    toggleClass : function(elt, name)
    {
        if (this.hasClass(elt, name))
            this.removeClass(elt, name);
        else
            this.setClass(elt, name);
    },

    // ************************************************************************************************
    // DOM queries

    $ : function(id, doc)
    {
        if (doc)
            return doc.getElementById(id);
        else
            return document.getElementById(id);
    },

    getChildByClass : function(node)
    {
        for (var i = 1; i < arguments.length; ++i)
        {
            var className = arguments[i];
            var child = node.firstChild;
            node = null;
            for (; child; child = child.nextSibling)
            {
                if (this.hasClass(child, className))
                {
                    node = child;
                    break;
                }
            }
        }

        return node;
    },

    getAncestorByClass : function(node, className)
    {
        for (var parent = node; parent; parent = parent.parentNode)
        {
            if (this.hasClass(parent, className))
                return parent;
        }

        return null;
    },

    getElementByClass : function(node, className)
    {
        for (var child = node.firstChild; child; child = child.nextSibling)
        {
            if (this.hasClass(child, className))
                return child;
            else
            {
                var found = this.getElementByClass(child, className);
                if (found)
                    return found;
            }
        }

        return null;
    },

    isAncestor : function(node, potentialAncestor)
    {
        for (var parent = node; parent; parent = parent.parentNode)
        {
            if (parent == potentialAncestor)
                return true;
        }

        return false;
    },

    getNextElement : function(node)
    {
        while (node && node.nodeType != 1)
            node = node.nextSibling;

        return node;
    },

    getPreviousElement : function(node)
    {
        while (node && node.nodeType != 1)
            node = node.previousSibling;

        return node;
    },

    getBody : function(doc)
    {
        if (doc.body)
            return doc.body;

        return doc.getElementsByTagName("body")[0];
    },

    findNextDown : function(node, criteria)
    {
        if (!node)
            return null;

        for (var child = node.firstChild; child; child = child.nextSibling)
        {
            if (criteria(child))
                return child;

            var next = this.findNextDown(child, criteria);
            if (next)
                return next;
        }
    },

    findPreviousUp : function(node, criteria)
    {
        if (!node)
            return null;

        for (var child = node.lastChild; child; child = child.previousSibling)
        {
            var next = this.findPreviousUp(child, criteria);
            if (next)
                return next;

            if (criteria(child))
                return child;
        }
    },

    findNext : function(node, criteria, upOnly, maxRoot)
    {
        if (!node)
            return null;

        if (!upOnly)
        {
            var next = this.findNextDown(node, criteria);
            if (next)
                return next;
        }

        for (var sib = node.nextSibling; sib; sib = sib.nextSibling)
        {
            if (criteria(sib))
                return sib;

            var next = this.findNextDown(sib, criteria);
            if (next)
                return next;
        }

        if (node.parentNode && node.parentNode != maxRoot)
            return this.findNext(node.parentNode, criteria, true);
    },

    findPrevious : function(node, criteria, downOnly, maxRoot)
    {
        if (!node)
            return null;

        for (var sib = node.previousSibling; sib; sib = sib.previousSibling)
        {
            var prev = this.findPreviousUp(sib, criteria);
            if (prev)
                return prev;

            if (criteria(sib))
                return sib;
        }

        if (!downOnly)
        {
            var next = this.findPreviousUp(node, criteria);
            if (next)
                return next;
        }

        if (node.parentNode && node.parentNode != maxRoot)
        {
            if (criteria(node.parentNode))
                return node.parentNode;

            return this.findPrevious(node.parentNode, criteria, true);
        }
    },

    getNextByClass : function(root, state)
    {
        function iter(node) { return node.nodeType == 1 && CopyURLLib.hasClass(node, state); }
        return this.findNext(root, iter);
    },

    getPreviousByClass : function(root, state)
    {
        function iter(node) { return node.nodeType == 1 && CopyURLLib.hasClass(node, state); }
        return this.findPrevious(root, iter);
    },

    hasChildElements : function(node)
    {
        if (node.contentDocument) // iframes
            return true;

        for (var child = node.firstChild; child; child = child.nextSibling)
        {
            if (child.nodeType == 1)
                return true;
        }

        return false;
    },

    isElement : function(o)
    {
        try {
            return o && o instanceof Element;
        }
        catch (ex) {
            return false;
        }
    },

    isNode : function(o)
    {
        try {
            return o && o instanceof Node;
        }
        catch (ex) {
            return false;
        }
    },

    // ************************************************************************************************
    // DOM Modification

    setOuterHTML : function(element, html)
    {
        var doc = element.ownerDocument;
        var range = doc.createRange();
        range.selectNode(doc.documentElement);

        var fragment = range.createContextualFragment(html);
        var first = fragment.firstChild;
        var last = fragment.lastChild;
        element.parentNode.replaceChild(fragment, element);
        return [first, last];
    },

    appendInnerHTML : function(element, html)
    {
        var doc = element.ownerDocument;
        var range = doc.createRange();
        range.selectNode(doc.body);

        var fragment = range.createContextualFragment(html);
        element.appendChild(fragment);
    },

    insertTextIntoElement : function(element, text)
    {
        var command = "cmd_insertText";

        var controller = element.controllers.getControllerForCommand(command);
        if (!controller || !controller.isCommandEnabled(command))
            return;

        var params = this.CCIN("@mozilla.org/embedcomp/command-params;1", "nsICommandParams");
        params.setStringValue("state_data", text);

        controller = this.QI(controller, this.CI("nsICommandController"));
        controller.doCommandWithParams(command, params);
    },

    // ************************************************************************************************
    // XPath

    /**
     * Gets an XPath for an element which describes its hierarchical location.
     */
    getElementXPath : function(element)
    {
        if (element && element.id)
            return '//*[@id="' + element.id + '"]';
        else
            return this.getElementTreeXPath(element);
    },

    getElementTreeXPath : function(element)
    {
        var paths = [];

        for (; element && element.nodeType == 1; element = element.parentNode)
        {
            var index = 0;
            for (var sibling = element.previousSibling; sibling; sibling = sibling.previousSibling)
            {
                if (sibling.localName == element.localName)
                    ++index;
            }

            var tagName = element.localName.toLowerCase();
            var pathIndex = (index ? "[" + (index+1) + "]" : "");
            paths.splice(0, 0, tagName + pathIndex);
        }

        return paths.length ? "/" + paths.join("/") : null;
    },

    cssToXPath : function(rule)
    {
        var regElement = /^([#.]?)([a-z0-9\\*_-]*)((\|)([a-z0-9\\*_-]*))?/i;
        var regAttr1 = /^\[([^\]]*)\]/i;
        var regAttr2 = /^\[\s*([^~=\s]+)\s*(~?=)\s*"([^"]+)"\s*\]/i;
        var regPseudo = /^:([a-z_-])+/i;
        var regCombinator = /^(\s*[>+\s])?/i;
        var regComma = /^\s*,/i;

        var index = 1;
        var parts = ["//", "*"];
        var lastRule = null;

        while (rule.length && rule != lastRule)
        {
            lastRule = rule;

            // Trim leading whitespace
            rule = rule.replace(/^\s*|\s*$/g,"");
            if (!rule.length)
                break;

            // Match the element identifier
            var m = regElement.exec(rule);
            if (m)
            {
                if (!m[1])
                {
                    // XXXjoe Namespace ignored for now
                    if (m[5])
                        parts[index] = m[5];
                    else
                        parts[index] = m[2];
                }
                else if (m[1] == '#')
                    parts.push("[@id='" + m[2] + "']");
                else if (m[1] == '.')
                    parts.push("[contains(@class, '" + m[2] + "')]");

                rule = rule.substr(m[0].length);
            }

            // Match attribute selectors
            m = regAttr2.exec(rule);
            if (m)
            {
                if (m[2] == "~=")
                    parts.push("[contains(@" + m[1] + ", '" + m[3] + "')]");
                else
                    parts.push("[@" + m[1] + "='" + m[3] + "']");

                rule = rule.substr(m[0].length);
            }
            else
            {
                m = regAttr1.exec(rule);
                if (m)
                {
                    parts.push("[@" + m[1] + "]");
                    rule = rule.substr(m[0].length);
                }
            }

            // Skip over pseudo-classes and pseudo-elements, which are of no use to us
            m = regPseudo.exec(rule);
            while (m)
            {
                rule = rule.substr(m[0].length);
                m = regPseudo.exec(rule);
            }

            // Match combinators
            m = regCombinator.exec(rule);
            if (m && m[0].length)
            {
                if (m[0].indexOf(">") != -1)
                    parts.push("/");
                else if (m[0].indexOf("+") != -1)
                    parts.push("/following-sibling::");
                else
                    parts.push("//");

                index = parts.length;
                parts.push("*");
                rule = rule.substr(m[0].length);
            }

            m = regComma.exec(rule);
            if (m)
            {
                parts.push(" | ", "//", "*");
                index = parts.length-1;
                rule = rule.substr(m[0].length);
            }
        }

        var xpath = parts.join("");
        return xpath;
    },

    getElementsBySelector : function(doc, css)
    {
        var xpath = this.cssToXPath(css);
        return this.getElementsByXPath(doc, xpath);
    },

    getElementsByXPath : function(doc, xpath)
    {
        var nodes = [];

        try {
            var result = doc.evaluate(xpath, doc, null, XPathResult.ANY_TYPE, null);
            for (var item = result.iterateNext(); item; item = result.iterateNext())
                nodes.push(item);
        }
        catch (exc)
        {
            // Invalid xpath expressions make their way here sometimes.  If that happens,
            // we still want to return an empty set without an exception.
        }

        return nodes;
    },

    getRuleMatchingElements : function(rule, doc)
    {
        var css = rule.selectorText;
        var xpath = this.cssToXPath(css);
        return this.getElementsByXPath(doc, xpath);
    },

    // Removes all child elements from an element
    removeAllChildElements : function(element)
    {
        // If the element is set
        if(element)
        {
            var childElements = element.childNodes;

            // Loop through the child elements
            for(var i = 0; i < childElements.length; i++)
            {
                element.removeChild(childElements[i]);
            }

            childElements = element.childNodes;

            // Loop through the child elements
            while(childElements.length > 0)
            {
                element.removeChild(childElements[0]);
            }
        }
    }
};