var CopyURLOptions = {
    $ : function(id)
    {
        return CopyURLLib.$(id, null);
    },

    Log : function (message)
    {
        CopyURLUtilities.Log(String.prototype.format.apply(message, Array.slice(arguments, 1)));
    },
    
    SaveOptions : function() {
        var copyURL_selectedFormat = this.$("copyURL-selectedFormat");
        
        if(copyURL_selectedFormat.selectedIndex == -1) {
            alert("Please select a format to use.");
            return false;
        }
        
        CopyURLPrefs.SetPreference("selectedFormat", copyURL_selectedFormat.selectedIndex + 1);

        this.SaveFormats();
        return true;
    },
    
    ResetPreferenceByObject : function(objIDWithPref) {
        var objWithPref = this.$(objIDWithPref);
        
        if(null == objWithPref)
        {
            return;
        }   
        
        var preferenceToReset = this.$(objWithPref.getAttribute("preference"));
        
        if(null == preferenceToReset)
        {
            return;
        }   
        
        try  {
            preferenceToReset.reset();
        } catch(e) {
            // normal to throw if user hasn't set a value
        }
    },
    
    InitializeFormats : function() {
        var copyURLFormats_listbox = this.$("copyURLFormats-listbox");
        var copyURL_selectedFormat = this.$("copyURL-selectedFormat");
        
        var formatCount = CopyURLPrefs.GetPreference("formatCount");
        var selectedFormat = CopyURLPrefs.GetPreference("selectedFormat");
        
        for(var i = 1; i <= formatCount; i++) {
            var name = CopyURLPrefs.GetPreference("formats.{0}.name".format(i));
            var accesskey = CopyURLPrefs.GetPreference("formats.{0}.accesskey".format(i));
            var linkformat = CopyURLPrefs.GetPreference("formats.{0}.linkformat".format(i));
            var imageformat = CopyURLPrefs.GetPreference("formats.{0}.imageformat".format(i));

            this.AddFormatToListBox(copyURLFormats_listbox, name, accesskey, linkformat, imageformat);
            this.AddFormatToSelectedFormatMenu(copyURL_selectedFormat, name, i, selectedFormat);
        }

        CopyURLOptions.FormatsListBoxSelected();
    },
    
    FormatsListBoxSelected : function() {
        var deleteButton = this.$("copyURLFormats-delete");
        var editButton = this.$("copyURLFormats-edit");
        var listBox = this.$("copyURLFormats-listbox");
        var selectedIndex = listBox.selectedIndex;

        // If an item is selected, enable edit/delete
        deleteButton.disabled = !listBox.selectedItem || (listBox.getRowCount() <= 1);
        editButton.disabled = !listBox.selectedItem;
    },
    
    AddFormatToSelectedFormatMenu : function(menulist, name, i, selectedFormat) {
        var menuitem = menulist.insertItemAt(i-1, name, i);
        
        if(i == selectedFormat) {
            menulist.selectedIndex = i-1;
        }
    },
    
    AddFormatToListBox : function(listbox, name, accesskey, linkformat, imageformat) {
        var listItem = document.createElement("listitem");
        
        this.BuildFormatRow(listItem, name, accesskey, linkformat, imageformat);

        return listbox.appendChild(listItem);
    },
    
    BuildFormatRow : function(listItem, name, accesskey, linkformat, imageformat) {
        var columns = [name, accesskey, linkformat, imageformat];
        
        for(column in columns) {
            var listCell = document.createElement("listcell");
            listCell.setAttribute("label", columns[column]);
            
            listItem.appendChild(listCell);
        }
    },
    
    AddFormat : function() {
        var params = {input : { action : "add", title : "Add New Format", name : "", accesskey : "", linkformat : "", imageformat : ""}, output : null };
        window.openDialog("chrome://copyurl/content/options/dialogs/format.xul", "copyURLFormats-dialog", "centerscreen,chrome,modal", params).focus();

        if(!params.output) return;

        var copyURLFormats_listbox = this.$("copyURLFormats-listbox");
        var copyURL_selectedFormat = this.$("copyURL-selectedFormat");
        var selectedFormat = CopyURLPrefs.GetPreference("selectedFormat");
        
        var addedIndex = this.AddFormatToListBox(copyURLFormats_listbox, params.output.name, params.output.accesskey, params.output.linkformat, params.output.imageformat);

        copyURLFormats_listbox.selectItem(addedIndex);
        copyURLFormats_listbox.ensureElementIsVisible(copyURLFormats_listbox.selectedItem);
        
        this.AddFormatToSelectedFormatMenu(copyURL_selectedFormat, params.output.name, addedIndex, selectedFormat);
    },
    
    EditFormat : function() {
        var copyURLFormats_listbox = this.$("copyURLFormats-listbox");
        var copyURL_selectedFormat = this.$("copyURL-selectedFormat");

        var selectedItem = copyURLFormats_listbox.selectedItem;

        // If an option is selected
        if(!selectedItem) return;

        var associatedMenuItem = copyURL_selectedFormat.getItemAtIndex(copyURLFormats_listbox.selectedIndex);

        var params = {
            input : { 
                action : "edit", 
                title : "Edit Existing Format", 
                name : selectedItem.childNodes[0].getAttribute("label"), 
                accesskey : selectedItem.childNodes[1].getAttribute("label"), 
                linkformat : selectedItem.childNodes[2].getAttribute("label"), 
                imageformat : selectedItem.childNodes[3].getAttribute("label")
            }, output : null };
            
        window.openDialog("chrome://copyurl/content/options/dialogs/format.xul", "copyURLFormats-dialog", "centerscreen,chrome,modal", params).focus();
        
        if(!params.output) return;

        while(selectedItem.hasChildNodes()) {
            selectedItem.removeChild(selectedItem.firstChild);
        }
        
        this.BuildFormatRow(selectedItem, params.output.name, params.output.accesskey, params.output.linkformat, params.output.imageformat);

        copyURLFormats_listbox.ensureElementIsVisible(selectedItem);
        
        associatedMenuItem.setAttribute("label", params.output.name);
    },

    DeleteFormat : function() {
        var copyURLFormats_listbox = this.$("copyURLFormats-listbox");
        var copyURL_selectedFormat = this.$("copyURL-selectedFormat");
        var selectedFormat = CopyURLPrefs.GetPreference("selectedFormat");
        
        var selectedItem = copyURLFormats_listbox.selectedItem;
        var selectedIndex = copyURLFormats_listbox.selectedIndex;

        // If an item is selected and it is a separator or the deletion is confirmed
        if(selectedItem && this.ConfirmDelete("Delete this Format?", "Are you sure you want to delete this URL format?")) {
            copyURLFormats_listbox.removeChild(selectedItem);
            copyURL_selectedFormat.removeItemAt(selectedIndex);
            
            if((selectedIndex + 1) == selectedFormat) {
                // it's gone, don't assume what they want to select
                copyURL_selectedFormat.selectedIndex = -1;
            }
        }
    },
    
    ConfirmDelete : function(title, message) {
        var promptOnDeletePref = "promptOnDeleteFormat";
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        var promptOnFormatDelete = CopyURLPrefs.GetPreference(promptOnDeletePref);

        if(promptOnFormatDelete != false) {
            var checkBox = {value : false};

            var ConfirmDelete = (promptService.confirmCheck(window, title, message, "Do not ask me again", checkBox) != 0);
            CopyURLPrefs.SetPreference(promptOnDeletePref, !checkBox.value);

            return ConfirmDelete;
        }

        return true;
    },
    
    ConfirmReset : function(title, message) {
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        return promptService.confirm(window, title, message) != 0;
    },
    
    ResetFormats : function() {
        if(!this.ConfirmReset("Reset formats to default?", "Are you sure you want to delete all existing formats and restore the defaults?")) {
            return;
        }
        
        var formatCount = CopyURLPrefs.GetPreference("formatCount");
        
        for(i = 1; i <= formatCount; i++) {
            CopyURLPrefs.DeletePreference("formats.{0}.name".format(i));
            CopyURLPrefs.DeletePreference("formats.{0}.accesskey".format(i));
            CopyURLPrefs.DeletePreference("formats.{0}.linkformat".format(i));
            CopyURLPrefs.DeletePreference("formats.{0}.imageformat".format(i));
        }
        
        CopyURLPrefs.DeletePreference("formatCount");
        CopyURLPrefs.DeletePreference("selectedCount");
        
        var prefsToReset = [
            "CopyURLPrefFormatCount", 
            "CopyURLPrefFormats_1_name", 
            "CopyURLPrefFormats_1_accesskey", 
            "CopyURLPrefFormats_1_linkformat", 
            "CopyURLPrefFormats_1_imageformat", 
            "CopyURLPrefFormats_2_name", 
            "CopyURLPrefFormats_2_accesskey", 
            "CopyURLPrefFormats_2_linkformat", 
            "CopyURLPrefFormats_2_imageformat", 
            "CopyURLPrefFormats_3_name", 
            "CopyURLPrefFormats_3_accesskey", 
            "CopyURLPrefFormats_3_linkformat", 
            "CopyURLPrefFormats_3_imageformat", 
            "CopyURLPrefSelectedFormat"];
        
        CopyURLUtilities.each(function(pref){
            CopyURLOptions.$(pref).reset();
        }, prefsToReset);
        
        var copyURLFormats_listbox = this.$("copyURLFormats-listbox");
        
        while(copyURLFormats_listbox.getRowCount() > 0) {
            copyURLFormats_listbox.removeItemAt(0);
        }
        
        var copyURL_selectedFormat = this.$("copyURL-selectedFormat");
        copyURL_selectedFormat.removeAllItems();
        
        this.InitializeFormats();
    },
    
    SaveFormats : function () {
        var copyURLFormats_listbox = this.$("copyURLFormats-listbox");
        var copyURLFormatsCount = copyURLFormats_listbox.getRowCount();
        var copyURLFormatsPreviousCount = CopyURLPrefs.GetPreference("formatCount");

        if("undefined" == copyURLFormatsPreviousCount) { copyURLFormatsPreviousCount = 0; }

        var newFormats = {};

        for(i = 0; i < copyURLFormatsCount; i++) {
            var listItem = copyURLFormats_listbox.getItemAtIndex(i);
            var copyURLFormats_index = i+1;
            
            newFormats["formats.{0}.name".format(copyURLFormats_index)] = listItem.childNodes[0].getAttribute("label").trim();
            newFormats["formats.{0}.accesskey".format(copyURLFormats_index)] = listItem.childNodes[1].getAttribute("label").trim();
            newFormats["formats.{0}.linkformat".format(copyURLFormats_index)] = listItem.childNodes[2].getAttribute("label").trim();
            newFormats["formats.{0}.imageformat".format(copyURLFormats_index)] = listItem.childNodes[3].getAttribute("label").trim();
        }

        for(i = 1; i <= copyURLFormatsPreviousCount; i++) {
            CopyURLPrefs.DeletePreference("formats.{0}.name".format(i));
            CopyURLPrefs.DeletePreference("formats.{0}.accesskey".format(i));
            CopyURLPrefs.DeletePreference("formats.{0}.linkformat".format(i));
            CopyURLPrefs.DeletePreference("formats.{0}.imageformat".format(i));
        }

        for(format in newFormats) {
            CopyURLPrefs.SetOrCreateStringPreference(format, newFormats[format]);
        }

        var rowCount = copyURLFormats_listbox.getRowCount()
        CopyURLPrefs.SetPreference('formatCount', rowCount - 1); // Bullshit, but it's the only way the preference observer will catch edits
        CopyURLPrefs.SetPreference('formatCount', rowCount);
    },

};