var CopyURLPrefs = {
    preferencePrefix : "extensions.CopyURL.",

    preferences : null,
    initialized : false,

    InitializePreferences : function() {
        if(!this.initialized || null == this.preferences)
        {
            this.preferences = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(this.preferencePrefix);
            this.initialized = true;
        }

        if(null == this.preferences)
        {
            this.initialized = false;
        }

        return this.initialized;
    },

    // TODO: Add default return value
    GetPreference : function(preferenceName) {
        if(!this.InitializePreferences()) { throw "Unable to initialize preferences."; }

        try {
            switch(this.preferences.getPrefType(preferenceName))
            {
                case this.preferences.PREF_STRING:
                {
                    return this.preferences.getCharPref(preferenceName);
                }
                case this.preferences.PREF_INT:
                {
                    return this.preferences.getIntPref(preferenceName);
                }
                case this.preferences.PREF_BOOL:
                {
                    return this.preferences.getBoolPref(preferenceName);
                }
            }
        }
        finally {
            // Do nothing, but we want the exception to be thrown up the chain.
        }
    },

    // SetPreference(preferenceName, value) - Magically set a preference
    SetPreference : function(preferenceName, value) {
        if(!this.InitializePreferences()) { throw "Unable to initialize preferences."; }

        try {
            switch(this.preferences.getPrefType(preferenceName))
            {
                case this.preferences.PREF_STRING:
                {
                    this.preferences.setCharPref(preferenceName, value);
                    return;
                }
                case this.preferences.PREF_INT:
                {
                    this.preferences.setIntPref(preferenceName, value);
                    return;
                }
                case this.preferences.PREF_BOOL:
                {
                    this.preferences.setBoolPref(preferenceName, value);
                    return;
                }
            }
        }
        finally {
            // Do nothing, but we want the exception to be thrown up the chain.
        }
    },

    // Sets a string preference
    SetOrCreateStringPreference : function(preferenceName, value)
    {
        // If the preference is set
        if(preferenceName)
        {
            var supportsStringInterface = Components.interfaces.nsISupportsString;
            var string                  = Components.classes["@mozilla.org/supports-string;1"].createInstance(supportsStringInterface);

            string.data = value;

            this.preferences.setComplexValue(preferenceName, supportsStringInterface, string);
        }
    },

    DeletePreference : function(preferenceName) {
        if(preferenceName && this.preferences && this.preferences.prefHasUserValue(preferenceName))
        {
            this.preferences.clearUserPref(preferenceName);
        }
    },

    GetBrowserPreference : function(preferenceName) {
        var browser_prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

        try {
            switch(browser_prefs.getPrefType(preferenceName))
            {
                case browser_prefs.PREF_STRING:
                {
                    return browser_prefs.getCharPref(preferenceName);
                }
                case browser_prefs.PREF_INT:
                {
                    return browser_prefs.getIntPref(preferenceName);
                }
                case browser_prefs.PREF_BOOL:
                {
                    return browser_prefs.getBoolPref(preferenceName);
                }
            }
        }
        finally {
            // Do nothing, but we want the exception to be thrown up the chain.
        }
    },
};

window.addEventListener("load", function(e) { CopyURLPrefs.InitializePreferences(e); }, false);
