String.prototype.trim = function() {
	return this.replace(/^\s+|\s+$/g,"");
}

String.prototype.ltrim = function() {
	return this.replace(/^\s+/,"");
}

String.prototype.rtrim = function() {
	return this.replace(/\s+$/,"");
}

String.prototype.is_blank = function() { 
    return null == this || this.trim().length == 0;
}

String.prototype.format = function() {
    var argLen = arguments.length;

    if( argLen == 0 ) return this;

    var str = this;
    for(var i = 0; i < argLen; i++)
    {
        var re = new RegExp('\\{' + i + '\\}','gm');
        str = str.replace(re, arguments[i]);
    }

    return str;
}

var CopyURLUtilities = {
    logService : null,
    logPrefix : null,
    debugging : false,

    Initialize : function(log_prefix) {
        this.debugging = CopyURLPrefs.GetPreference("debug");

        this.logPrefix = log_prefix;

        if (null == this.logService)
        {
            this.logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
            this.logService.QueryInterface(Components.interfaces.nsIConsoleService);
        }
    },

    Log : function(message) {
        if(this.debugging)
        {
            this.Initialize(this.logPrefix);
            this.logService.logStringMessage("{0}: {1}".format(this.logPrefix, message));
        }
    },

    toggleDebugMode : function() {
        this.debugging = !this.debugging;
        this.SetPreference("debug", this.debugging);
    },

    openURL : function(url, sameTab, openInBackground) {
        if(sameTab)
        {
            top.content.document.location = url;
        }
        else
        {
            var oldTab = getBrowser().selectedTab;
            var newTab = getBrowser().addTab(url);

            getBrowser().selectedTab = openInBackground ? oldTab : newTab;
        }
    },

    // Set(variable args) - Returns a hash of the arguments passed in, for use in set operations
    // Example:
    // var nodeType = 2;
    // var typeInSet = nodeType in CopyURLUtilities.Set(2,4,6) => true
    // var typeInSet = nodeType in CopyURLUtilities.Set(1,3,5) => false
    Set : function() {
        var result = {};

        for(var i = 0, len = arguments.length; i < len; i++)
        {
            if(arguments[i] instanceof Array)
            {
                for(var j = 0, i_len = arguments[i].length; j < i_len; j++)
                {
                    result[arguments[i][j]] = true;
                }
            }
            else
            {
                result[arguments[i]] = true;
            }
        }

        return result;
    },

    // map(func, list) - Applies function func to every element of list, overwriting the current element.
    // Example:
    // list=[1,2,3,4,5];
    // CopyURLUtilities.map(function(x) { return x*2; }, list); => [2,4,6,8,10]
    // CopyURLUtilities.map(function(x) { return x*x; }, list); => [1,4,9,16,25]
    map : function (func, list) {
        var l = list.length;
        var ret = [];

        for(var i = 0; i < l; i++)
        {
            ret[i] = func(list[i]);
        }

        return ret;
    },

    // filter(func, list) - Returns a new list containing elements from the given list who meet the conditions in the filtering function func.
    // Example:
    // list = [1,2,3,4,5];
    // CopyURLUtilities.filter(function(x) { return x % 2 == 1; }, list); => [1,3,5]
    // CopyURLUtilities.filter(function(x) { return x % 2 == 0; }, list); => [2,4]
    // CopyURLUtilities.filter(function(x) { return x >= 3; }, list); => [3,4,5]
    filter : function (func, list) {
        var l = list.length;
        var ret = [];

        for(var i = 0; i < l; i++)
        {
            if(true == func(list[i]))
            {
                ret[ret.length] = list[i];
            }
        }

        return ret;
    },

    // reduce(func, list, init) - Compresses a list down to a single value, by performing function func on each element combined with an accumulator, initialized by init.
    // Example:
    // list = [1,2,3,4,5];
    // CopyURLUtilities.reduce(function(x,y){return x+y;}, list, 0); => 15
    // CopyURLUtilities.reduce(function(x,y){return x+y;}, list, ""); => "12345"
    // CopyURLUtilities.reduce(function(x,y){return x*y;}, list, 1); => 120 (notice accumulator starts at 1 for multiplication, since 0 would always return 0
    reduce : function (func, list, init) {
        var l = list.length;
        var last = init;

        for(var i = 0; i < l; i++)
        {
            last = func(last, list[i]);
        }

        return last;
    },
    
    // each(func, list) - Performs the given function on each element of the given list
    each : function (func, list) {
        var l = list.length;
        
        for(var i = 0; i < l; i++)
        {
            func(list[i]);
        }
    },
    
    $ : function(id)
    {
        return CopyURLLib.$(id, null);
    },

    HackTextboxesIntoWrappedLabels : function() {
        if(0 == arguments.length) return;
        
        CopyURLUtilities.each(function(textboxIDToTransform) {
            var textboxToTransform = CopyURLUtilities.$(textboxIDToTransform);
            
            if(null == textboxToTransform) return;
            
            textboxToTransform.setAttribute("class", "plain");
            textboxToTransform.setAttribute("style", "background-color: transparent;");
            textboxToTransform.setAttribute("flex", "1");
            textboxToTransform.setAttribute("multiline", "true");
            textboxToTransform.setAttribute("readonly", "true");
            textboxToTransform.setAttribute("onoverflow", "this.height = this.inputField.scrollHeight;");
            textboxToTransform.setAttribute("DOMAttrModified", "if(event.attrName == 'value') this.value = event.newValue; return true;");
        }, arguments);
    },
};

window.addEventListener("load", function(e) { CopyURLUtilities.Initialize("CopyURL"); }, false);
